// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright (c) 2015 Google, Inc
 * Written by Simon Glass <sjg@chromium.org>
 */

#include <common.h>
#include <dm.h>
#include <errno.h>
#include <led.h>
#include <asm/gpio.h>
#include <dm/lists.h>
#include <pwm.h>
#include <mailbox.h>

#include <debug_impl.h>

struct led_pwm_priv {
    struct udevice *pwm_dev;
    unsigned int pwm_led_index;
    unsigned int period_ms;
    unsigned int duty_ms;
    unsigned short invert;
    enum led_state_t state;
    struct mbox_chan mbox;
};


static int pwm_led_set_state(struct udevice *dev, enum led_state_t state)
{
    struct led_pwm_priv *priv = dev_get_priv(dev);
    int ret;
    u8 send_box[10] = {0};
    UBOOT_DEBUG("\n ##state:%d\n",state);

    if (priv->pwm_dev == NULL)
        return -EREMOTEIO;
    switch (state) {
    case LEDST_OFF:
        priv->duty_ms = 0;
        break;
    case LEDST_ON:
        priv->duty_ms = priv->period_ms;
        break;
    case LEDST_TOGGLE:
        if(priv->state == LEDST_OFF)
            priv->duty_ms = priv->period_ms;
        else if(priv->state == LEDST_ON)
            priv->duty_ms = 0;
        break;
#ifdef CONFIG_LED_BLINK
    case LEDST_BLINK:
        priv->duty_ms = priv->period_ms/2;
        break;
#endif
    case LEDST_LOOP:
    {
        send_box[0] = LEDST_LOOP;
        ret = mbox_send(&priv->mbox, send_box);
        UBOOT_DEBUG("%s: mbox_send : %d\n",__func__, ret);
        if (ret) {
           UBOOT_DEBUG("%s: mbox_send failed: %d\n",__func__, ret);
        }
        return ret;
    }
    case LEDST_LIGHT:
    {
        send_box[0] = LEDST_LIGHT;
        ret = mbox_send(&priv->mbox, send_box);
        UBOOT_DEBUG("%s: mbox_send : %d\n",__func__, ret);
        if (ret) {
           UBOOT_DEBUG("%s: mbox_send failed: %d\n",__func__, ret);
        }
        return ret;
    }
    case LEDST_ONE_SHOT:
    {
        send_box[0] = LEDST_ONE_SHOT;
        ret = mbox_send(&priv->mbox, send_box);
        UBOOT_DEBUG("%s: mbox_send : %d\n",__func__, ret);
        if (ret) {
           UBOOT_DEBUG("%s: mbox_send failed: %d\n",__func__, ret);
        }
        return ret;
    }
    default:
        return -ENOSYS;
    }

    ret = pwm_set_enable(priv->pwm_dev, priv->pwm_led_index, 0);
    if (ret < 0)
        return ret;

    UBOOT_DEBUG("\nperiod_ms = %d duty_ms:%d\n",priv->period_ms,priv->duty_ms);
    ret = pwm_set_config(priv->pwm_dev, priv->pwm_led_index, priv->period_ms*1000000, priv->duty_ms*1000000);
    if (ret < 0)
        return ret;

    ret = pwm_set_enable(priv->pwm_dev, priv->pwm_led_index, 1);
    priv->state = state;

    return ret;
}

static enum led_state_t pwm_led_get_state(struct udevice *dev)
{
    struct led_pwm_priv *priv = dev_get_priv(dev);

    return priv->state;
}

static int pwm_led_set_period(struct udevice *dev, int period_ms)
{
    struct led_pwm_priv *priv = dev_get_priv(dev);

    if (priv->pwm_dev == NULL)
        return -EREMOTEIO;

    priv->period_ms = period_ms;

    UBOOT_DEBUG("\nPeriod = %dms \n",priv->period_ms);

    return 0;
}


static int led_pwm_probe(struct udevice *dev)
{
    struct led_uc_plat *uc_plat = dev_get_uclass_platdata(dev);
    struct led_pwm_priv *priv = dev_get_priv(dev);
    int ret = 0;
    struct ofnode_phandle_args args;

    /* fill of args */
    args.args[0] = 61;
    args.args_count = 1;


    if (!uc_plat->label)
        return 0;

    UBOOT_DEBUG("%s: init label %s \n", __func__, uc_plat->label);
    ret |= uclass_get_device_by_name(UCLASS_PWM, "pwm@1c087800", &priv->pwm_dev);
    UBOOT_DEBUG("%s: line %d\n", __func__, __LINE__);
    ret |= dev_read_u32(dev, "pwm_index", &priv->pwm_led_index);
    UBOOT_DEBUG("%s: index %d\n", __func__, priv->pwm_led_index);
    ret |= dev_read_u32(dev, "period_ms", &priv->period_ms);
    UBOOT_DEBUG("%s: period_ms %d\n", __func__, priv->period_ms);
    ret |= dev_read_u32(dev, "duty_ms", &priv->duty_ms);
    UBOOT_DEBUG("%s: duty_ms %d\n", __func__, priv->duty_ms);
    ret |= dev_read_u32(dev, "invert", (u32 *)&priv->invert);
    UBOOT_DEBUG("%s: invert %d\n", __func__, priv->invert);

    ret = mbox_get_by_dev_name("mailbox_to_pm51", &args, &priv->mbox);
    if (ret) {
        UBOOT_DEBUG("%s mbox_get_by_index failed: %d\n", __func__, ret);
    }

    //pwm_set_invert(priv->pwm_dev, priv->pwm_led_index, priv->invert);
    //pwm_set_enable(priv->pwm_dev, priv->pwm_led_index, 1);
    priv->state = LEDST_OFF;

    return ret;
}

static int led_pwm_bind(struct udevice *parent)
{
    struct udevice *dev;
    ofnode node;
    int ret;

    dev_for_each_subnode(node, parent) {
        struct led_uc_plat *uc_plat;
        const char *label;

        label = ofnode_read_string(node, "label");
        if (!label) {
            UBOOT_ERROR("%s: node %s has no label\n", __func__,
                 ofnode_get_name(node));
                return -EINVAL;
        }
        UBOOT_DEBUG("%s: label %s\n", __func__, label);
        ret = device_bind_driver_to_node(parent, "pwm_led",
            ofnode_get_name(node),
            node, &dev);
        if (ret)
            return ret;
        uc_plat = dev_get_uclass_platdata(dev);
        uc_plat->label = label;
    }

    return 0;
}

static const struct led_ops pwm_led_ops = {
    .set_state  = pwm_led_set_state,
    .get_state  = pwm_led_get_state,
    .set_period = pwm_led_set_period,
};

static const struct udevice_id led_pwm_ids[] = {
    { .compatible = "mediatek,mt5896-pwm-leds" },
    { }
};

U_BOOT_DRIVER(led_pwm) = {
    .name   = "pwm_led",
    .id     = UCLASS_LED,
    .of_match = led_pwm_ids,
    .ops    = &pwm_led_ops,
    .priv_auto_alloc_size = sizeof(struct led_pwm_priv),
    .bind   = led_pwm_bind,
    .probe  = led_pwm_probe,
};
